


// MISC ENEMY FLAGS
// 1  - created as an enemy from script (not in screen enemy list, and probably contains sub-enemies such as gleeoks)
// 2  -


ffc script CreateBossEnemy
{
	void run(int enemy_id, int n, int hp)
	{
		//if boss screen was cleareed already.
		if( (Screen->D[0] & 0x01) != 0 )
			Quit();

		if(n == 0)
			n = 1;

		npc e;
		for(int i = 0;i < n; i++)
		{
			e = Screen->CreateNPC(enemy_id);
			if(!e->isValid())
				continue;

			if(hp > 0)
				e->HP = hp;

			e->X = this->X;
			e->Y = this->Y;

			//Misc[15] is used as various flags
			e->Misc[15] |= 0x01;
		}

		while(e->isValid())
		{
			Waitframe();
		}

		//finally, check and see if all enemies are dead. If so, let the screen know about it.
		while(true)
		{
			Waitframes(8);
			if( GetNthValidScreenEnemy(1) == 0 )
			{
				Screen->D[0] |= 0x01;
				Quit();
			}
		}
	}
}


ffc script CreateEnemy
{
	void run(int enemy_id, int n, int hp)
	{
		if(n == 0)
			n = 1;

		for(int i = 0;i < n; i++)
		{
			npc e = Screen->CreateNPC(enemy_id);
			if(!e->isValid())
				continue;

			if(hp > 0)
				e->HP = hp;

			e->X = this->X;
			e->Y = this->Y;
			e->Misc[15] |= 0x01;
		}
	}
}


ffc script PlaceEnemy
{
	void run(int n, int hp)
	{
		Waitframes(4);

		if(n == 0)
		{
			n = GetThisFFCNumber(this);
			if(n == 0 || n > 10)
				Quit();
		}

		int index = GetNthValidScreenEnemy(n);
		if(index == 0)
		{
			Quit();
		}

		npc e = Screen->LoadNPC(index);
		if(!e->isValid())
			Quit();

		if(hp > 0)
			e->HP = hp;

		e->X = this->X;
		e->Y = this->Y;
	}
}


// ***ENEMY SPAWNER***
// ( Note - All arguments except D0 are optional. You can set any number
//   of these that you want to use and leave any other amount of them blank.
//   The script will automagically do the rest for you.)
// D0 - ENEMY ID
// D1 - NUMBER TO SPAWN      / MAX ALLOWED ON SCREEN AT A TIME  (use decimal place for max)
// D2 - DELAY BETWEEN SPAWNS / INITIAL DELAY (use decimal place for initial delay)
// D3 - MIN HP OF ENEMY      / MAX HP (use decimal place for max HP)
// D4 - SPAWN ENEMIES ON COMBO TYPES (of type) INSTEAD OF FFC POS.
// D5 - SPAWN ENEMIES ON COMBO FLAGS (of type) INSTEAD OF FFC POS.
// D6 ()
// D7 - FLAGS (these can be combined together by simple addition. ex; flags 1 & 4 equals 5)

// enemy spawner flags:
// 1  - counts as boss / don't spawn if boss was defeated previously.
// 2  - create dummy enemy for spawn duration. (hold shutters and secrets)
// 4  - spawn on all screen flags or combos instead of only one. (still counts as only one enemy!)
// 8  - spawn at random walkable location instead of ffc pos.
// 16 - spawn on top of Link instead of ffc pos.

ffc script EnemySpawner
{
	void run(int enemy_id, int n, int delay, int hp, int combo_id, int combo_flag, int unused2, int flags)
	{
		if(enemy_id == 0)
			Quit();

		int max_n = GetRemainderAsInt(n);
		int max_hp = GetRemainderAsInt(hp);
		int start_delay = GetRemainderAsInt(delay);

		hp = hp << 0; //truncate
		if(max_hp == 0)
			max_hp = hp;

		delay = delay << 0;
		if(delay == 0)
			delay = 60;

		n = n << 0;
		if(n == 0)
			n = 1;

		if(max_n == 0)
			max_n = 255;

		if( (flags & 0x01) != 0 ) //is boss enemy spawner
		{
			if( (Screen->D[0] & 0x01) != 0 )
				Quit();
		}

		npc dummy;
		if( (flags & 0x02) != 0 ) //create hold enemy
		{
			dummy = CreateDummyEnemy();
		}

		if(start_delay != 0)
			Waitframes(start_delay);

		while(true)
		{
			// sleep while there are more enemies on screen than supplied with D1(max enemy) variable.
			while(Screen->NumNPCs() >= max_n)
				Waitframe();

			int enemy_hp = 0;
			if(hp > 0 && hp != max_hp)
				enemy_hp = Rand(hp, max_hp);

			if(combo_flag > 0)
			{
				if( (flags & 0x04) != 0 )
				{
					SpawnEnemyOnAllComboFlags(enemy_id, combo_flag, enemy_hp);
				}
				else
				{
					SpawnEnemyOnComboFlag(enemy_id, combo_flag, enemy_hp);
				}
			}
			else if(combo_id > 0)
			{
				if( (flags & 0x04) != 0 )
				{
					SpawnEnemyOnAllCombosOfID(enemy_id, combo_id, enemy_hp);
				}
				else
				{
					SpawnEnemyOnComboID(enemy_id, combo_id, enemy_hp);
				}
			}
			else if( (flags & 0x08) != 0 ) //spawn on random walkable combo
			{
				int temp = 1;
				if(delay == 0) //optimization
				{
					temp = n;
					n = 1;
				}

				SpawnScreenEnemy(enemy_id, temp);
			}
			else if( (flags & 0x10) != 0 ) //spawn on top of the player
			{
				npc e = CreateNPCAt(enemy_id, GridX(Link->X), GridY(Link->Y));
				if(enemy_hp > 0)
					e->HP = enemy_hp;
			}
			else //spawn at this pos.
			{
				npc e = Screen->CreateNPC(enemy_id);
				e->X = this->X;
				e->Y = this->Y;

				if(enemy_hp > 0)
					e->HP = enemy_hp;
			}

			n--;
			if(n <= 0)
			{
				if( (flags & 0x02) != 0 && dummy->isValid() )
				{
					dummy->HP = 0;
					dummy->X = -512;
				}

				if( (flags & 0x01) != 0 )
				{
					//counts as boss enemy
					while(true)
					{
						Waitframes(8);
						if( GetNthValidScreenEnemy(1) == 0 )
						{
							Screen->D[0] |= 0x01;
							break;
						}
					}
				}

				Quit();
			}

			Waitframes(delay);
		} //while(true)
	}
}


//***ARMOS***
// D0 - id of enemy to create
// D1 - (optional) HP of created enemy
// D2 - (optional) Links proximity to trigger the Armos ( defaults to 16 pixels )
// D3 - (optional) Number of enemies to spawn ( defaults to one )

const int DEFAULT_ARMOS_RADIUS = 21; //17 * (2 - .707) magic constant. :P

ffc script Armos
{
	void run(int enemy_id, int hp, int radius, int n)
	{
		if(radius == 0)
			radius = DEFAULT_ARMOS_RADIUS;

		n = Clamp(n, 1, 254);

		while(true)
		{
			Waitframe();
			if( Distance(this->X, Link->X, this->Y, Link->Y) < radius )
			{
				for( int i = n; i > 0; i-- )
				{
					npc armos = Screen->CreateNPC(enemy_id);
					armos->X = this->X;
					armos->Y = this->Y;
					armos->Misc[15] |= 0x01;

					if( hp != 0 )
						armos->HP = hp;
				}

				this->Data = 0;
				Quit();
			}
		}
	}
}



